@@ -6,6 +6,9 @@ BSD code
 Programmed by Bastian Ballmann
 Last update: 09.02.2006
 
+Fixed for FreeBSD by Niels Bakker
+Last update: 2007-12-26
+
 This program is free software; you can redistribute 
 it and/or modify it under the terms of the 
 GNU General Public License version 2 as published 
@@ -19,13 +22,18 @@ See the GNU General Public License for more details.
 */
 
 #include <stdio.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <ifaddrs.h>
 
+#include <net/if_dl.h>
+#include <net/if_types.h>
+#include <net/ethernet.h>
+
 int get_mac_bsd(u_char *dev, char *mac)
 {
-  struct ifaddrs *iface;
+  struct ifaddrs *iface, *iffirst;
 
   if(strlen(mac) > 0)
     strcpy(mac,"unknown");
@@ -35,28 +43,27 @@ int get_mac_bsd(u_char *dev, char *mac)
   if(strlen(dev) == 0)
     return -1;
 
-  strcpy(mac,"unkown");
-  getifaddrs(&iface);
-  
+  strcpy(mac,"unknown");
+  if (getifaddrs(&iface))
+    return -1;
+
+  iffirst = iface;
   while(iface->ifa_next != NULL)
-    { 
+    {
       if(!strcmp(iface->ifa_name,dev))
 	{
-	  if(iface->ifa_addr->sa_family != AF_INET)
+	  if (iface->ifa_addr->sa_family == AF_LINK)
 	    {
-	      sprintf(mac,"%02x:%02x:%02x:%02x:%02x:%02x",	
-		      iface->ifa_addr->sa_data[0] & 0xff, 
-		      iface->ifa_addr->sa_data[1] & 0xff,
-		      iface->ifa_addr->sa_data[2] & 0xff, 
-		      iface->ifa_addr->sa_data[3] & 0xff,
-		      iface->ifa_addr->sa_data[4] & 0xff, 
-		      iface->ifa_addr->sa_data[5] & 0xff);
+	      struct sockaddr_dl *sdl = (struct sockaddr_dl *) iface->ifa_addr;
+	      sprintf(mac, "%s",
+		      ether_ntoa((struct ether_addr *)LLADDR(sdl)));
 	      break;
 	    }
 	}
-      
+
       iface = iface->ifa_next;	
     }
 
+  freeifaddrs(iffirst);
   return 0;
 }
